@@ -237,11 +237,11 @@ func TestSimplifiedURLs(t *testing.T) {
 				"host4:1234",
 				"host5:",
 				"host6",
-				"nats://[1:2:3:4]:1234",
-				"nats://[5:6:7:8]:",
-				"nats://[9:10:11:12]",
-				"[13:14:15:16]:",
-				"[17:18:19:20]:1234",
+				"nats://[::1:2:3:4]:1234",
+				"nats://[::5:6:7:8]:",
+				"nats://[::9:10:11:12]",
+				"[::13:14:15:16]:",
+				"[::17:18:19:20]:1234",
 			},
 			[]string{
 				"nats://host1:1234/",
@@ -251,11 +251,11 @@ func TestSimplifiedURLs(t *testing.T) {
 				"nats://host4:1234",
 				"nats://host5:4222",
 				"nats://host6:4222",
-				"nats://[1:2:3:4]:1234",
-				"nats://[5:6:7:8]:4222",
-				"nats://[9:10:11:12]:4222",
-				"nats://[13:14:15:16]:4222",
-				"nats://[17:18:19:20]:1234",
+				"nats://[::1:2:3:4]:1234",
+				"nats://[::5:6:7:8]:4222",
+				"nats://[::9:10:11:12]:4222",
+				"nats://[::13:14:15:16]:4222",
+				"nats://[::17:18:19:20]:1234",
 			},
 		},
 		{
@@ -264,17 +264,17 @@ func TestSimplifiedURLs(t *testing.T) {
 				"ws://host1:1234",
 				"ws://host2:",
 				"ws://host3",
-				"ws://[1:2:3:4]:1234",
-				"ws://[5:6:7:8]:",
-				"ws://[9:10:11:12]",
+				"ws://[::1:2:3:4]:1234",
+				"ws://[::5:6:7:8]:",
+				"ws://[::9:10:11:12]",
 			},
 			[]string{
 				"ws://host1:1234",
 				"ws://host2:80",
 				"ws://host3:80",
-				"ws://[1:2:3:4]:1234",
-				"ws://[5:6:7:8]:80",
-				"ws://[9:10:11:12]:80",
+				"ws://[::1:2:3:4]:1234",
+				"ws://[::5:6:7:8]:80",
+				"ws://[::9:10:11:12]:80",
 			},
 		},
 		{
@@ -283,17 +283,17 @@ func TestSimplifiedURLs(t *testing.T) {
 				"wss://host1:1234",
 				"wss://host2:",
 				"wss://host3",
-				"wss://[1:2:3:4]:1234",
-				"wss://[5:6:7:8]:",
-				"wss://[9:10:11:12]",
+				"wss://[::1:2:3:4]:1234",
+				"wss://[::5:6:7:8]:",
+				"wss://[::9:10:11:12]",
 			},
 			[]string{
 				"wss://host1:1234",
 				"wss://host2:443",
 				"wss://host3:443",
-				"wss://[1:2:3:4]:1234",
-				"wss://[5:6:7:8]:443",
-				"wss://[9:10:11:12]:443",
+				"wss://[::1:2:3:4]:1234",
+				"wss://[::5:6:7:8]:443",
+				"wss://[::9:10:11:12]:443",
 			},
 		},
 	} {
